Long used destructors kfree_skb and kfree_skb_reason do not pass
receiving socket to packet drop tracepoints trace_kfree_skb.
This makes it hard to track packet drops of a certain netns (container)
or a socket (user application).

The naming of these destructors are also not consistent with most sk/skb
operating functions, i.e. functions named "sk_xxx" or "skb_xxx".
Introduce a new functions sk_skb_reason_drop as drop-in replacement for
kfree_skb_reason on local receiving path. Callers can now pass receiving
sockets to the tracepoints.

kfree_skb and kfree_skb_reason are still usable but they are now just
inline helpers that call sk_skb_reason_drop.

Note it is not feasible to do the same to consume_skb. Packets not
dropped can flow through multiple receive handlers, and have multiple
receiving sockets. Leave it untouched for now.

Suggested-by: Eric Dumazet <eduma...@google.com>
Signed-off-by: Yan Zhai <y...@cloudflare.com>
---
v1->v2: changes function names to be more consistent with common sk/skb
operations
---
 include/linux/skbuff.h | 10 ++++++++--
 net/core/skbuff.c      | 22 ++++++++++++----------
 2 files changed, 20 insertions(+), 12 deletions(-)

diff --git a/include/linux/skbuff.h b/include/linux/skbuff.h
index fe7d8dbef77e..c479a2515a62 100644
--- a/include/linux/skbuff.h
+++ b/include/linux/skbuff.h
@@ -1251,8 +1251,14 @@ static inline bool skb_data_unref(const struct sk_buff 
*skb,
        return true;
 }
 
-void __fix_address
-kfree_skb_reason(struct sk_buff *skb, enum skb_drop_reason reason);
+void __fix_address sk_skb_reason_drop(struct sock *sk, struct sk_buff *skb,
+                                     enum skb_drop_reason reason);
+
+static inline void
+kfree_skb_reason(struct sk_buff *skb, enum skb_drop_reason reason)
+{
+       sk_skb_reason_drop(NULL, skb, reason);
+}
 
 /**
  *     kfree_skb - free an sk_buff with 'NOT_SPECIFIED' reason
diff --git a/net/core/skbuff.c b/net/core/skbuff.c
index 2854afdd713f..9def11fe42c4 100644
--- a/net/core/skbuff.c
+++ b/net/core/skbuff.c
@@ -1190,7 +1190,8 @@ void __kfree_skb(struct sk_buff *skb)
 EXPORT_SYMBOL(__kfree_skb);
 
 static __always_inline
-bool __kfree_skb_reason(struct sk_buff *skb, enum skb_drop_reason reason)
+bool __sk_skb_reason_drop(struct sock *sk, struct sk_buff *skb,
+                         enum skb_drop_reason reason)
 {
        if (unlikely(!skb_unref(skb)))
                return false;
@@ -1203,26 +1204,27 @@ bool __kfree_skb_reason(struct sk_buff *skb, enum 
skb_drop_reason reason)
        if (reason == SKB_CONSUMED)
                trace_consume_skb(skb, __builtin_return_address(0));
        else
-               trace_kfree_skb(skb, __builtin_return_address(0), reason, NULL);
+               trace_kfree_skb(skb, __builtin_return_address(0), reason, sk);
        return true;
 }
 
 /**
  *     kfree_skb_reason - free an sk_buff with special reason
  *     @skb: buffer to free
  *     @reason: reason why this skb is dropped
  *
- *     Drop a reference to the buffer and free it if the usage count has
- *     hit zero. Meanwhile, pass the drop reason to 'kfree_skb'
- *     tracepoint.
+ *     Drop a reference to the buffer and free it if the usage count has hit
+ *     zero. Meanwhile, pass the receiving socket and drop reason to
+ *     'kfree_skb' tracepoint.
  */
 void __fix_address
-kfree_skb_reason(struct sk_buff *skb, enum skb_drop_reason reason)
+sk_skb_reason_drop(struct sock *sk, struct sk_buff *skb, enum skb_drop_reason 
reason)
 {
-       if (__kfree_skb_reason(skb, reason))
+       if (__sk_skb_reason_drop(sk, skb, reason))
                __kfree_skb(skb);
 }
-EXPORT_SYMBOL(kfree_skb_reason);
+EXPORT_SYMBOL(sk_skb_reason_drop);
 
 #define KFREE_SKB_BULK_SIZE    16
 
@@ -1261,7 +1263,7 @@ kfree_skb_list_reason(struct sk_buff *segs, enum 
skb_drop_reason reason)
        while (segs) {
                struct sk_buff *next = segs->next;
 
-               if (__kfree_skb_reason(segs, reason)) {
+               if (__sk_skb_reason_drop(NULL, segs, reason)) {
                        skb_poison_list(segs);
                        kfree_skb_add_bulk(segs, &sa, reason);
                }
@@ -1203,26 +1204,27 @@ bool __kfree_skb_reason(struct sk_buff *skb, enum 
 
 /**
- *     kfree_skb_reason - free an sk_buff with special reason
+ *     sk_skb_reason_drop - free an sk_buff with special reason
+ *     @sk: the socket to receive @skb, or NULL if not applicable
  *     @skb: buffer to free
  *     @reason: reason why this skb is dropped
-- 
2.30.2



Reply via email to